/****************************************************************************
 *
 * Project Title: Environmental Decision Support System
 *
 *         File: %W%
 *     Pathname: %P%
 * Last updated: %G% %U%
 *
 * Made available by MCNC and the Carolina Environmental Program of UNC Chapel
 * Hill under terms of the GNU Public License.  See gpl.txt for more details.
 *
 * See file COPYRIGHT for license information on this and supporting software.
 *
 * Carolina Environmental Program
 * University of North Carolina at Chapel Hill
 * 137 E. Franklin St.
 * Chapel Hill, NC 27599-6116
 *
 * See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
 *
 ****************************************************************************/
/*
MODIFICATION HISTORY:

WHO  WHEN       WHAT
---  ----       ----
SRT  09/18/95   Added lineWidth param to graph2d() routine
*/


static const char *SCCS_ID = "%W% %P% %G% %U%";

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include "nan_incl.h"

#define GRAPH2D_CMD "$BLT_WISH"
#define GRAPH2D_CFG "$EDSS_BINDIR/public_domain/blt/graph2d.blt"

#define PAVE_SUCCESS 1
#define FAILURE 0

	/* replaces any [ or ] chars with ( or ) chars, respectively */
void replaceBrackets(char *s)
{
int len, i;

if (!s) return;
len = strlen(s);
if (!len) return;
for (i = 0; i < len; i++)
	{
	if (s[i] == '[') s[i] = '(';
	if (s[i] == ']') s[i] = ')';
	}
return;
}


int graph2d(float *x, float *y, int nlines, int *npoints,
char *title, char *xaxis_label, char *yaxis_label, 
char **legend, char **symbol, char **color, char *message,
int lineWidth)
{
char *tmpfilename = NULL;
char *graph2d_cmd = NULL;
FILE *tcl_input;
int status;
int lw;
register int i, j, k;
pid_t pid;
char *graph2d_cfg;
static int num2d = 0;
FILE *cmdpipe;
#define BUFSIZE 256
static char response[BUFSIZE];
float value;
char *hlineval;

status = FAILURE;
replaceBrackets(title);
replaceBrackets(xaxis_label);
replaceBrackets(yaxis_label);

if ((tmpfilename = (char *) malloc(sizeof(char) * 512)) == NULL)
    goto ALLOCATE_GRAPH2D_ERROR;
if ((graph2d_cmd = (char *) malloc(sizeof(char) * 512)) == NULL)
    goto ALLOCATE_GRAPH2D_ERROR;

pid = getpid();
sprintf(tmpfilename, "/tmp/graph2d.blt.%d.%d", pid, num2d++);

graph2d_cfg = getenv("GRAPH2DCFG");
if (graph2d_cfg == NULL) {
  sprintf(graph2d_cmd, "%s -f %s < %s&", GRAPH2D_CMD, GRAPH2D_CFG, tmpfilename); 
}
else {
  sprintf(graph2d_cmd, "%s -f %s < %s&", GRAPH2D_CMD, graph2d_cfg, tmpfilename);
}

if ((tcl_input = fopen(tmpfilename, "w")) != NULL)
	{
	if (fprintf(tcl_input, "%s\n", tmpfilename) < 0)
		goto WRITE_GRAPH2D_ERROR;
	if (xaxis_label == NULL)
		{
		if (fprintf(tcl_input, "option add *graph.xTitle \"%s\"\n",
			 "X Axis Label") < 0)
			goto WRITE_GRAPH2D_ERROR;
		}
	else
		{
		if (fprintf(tcl_input, "option add *graph.xTitle \"%s\"\n",
			 xaxis_label) < 0)
			goto WRITE_GRAPH2D_ERROR;
		}
	if (yaxis_label == NULL)
		{
		if (fprintf(tcl_input, "option add *graph.yTitle \"%s\"\n",
			 "Y Axis Label") < 0)
			goto WRITE_GRAPH2D_ERROR;
		}
	else
		{
		if (fprintf(tcl_input, "option add *graph.yTitle \"%s\"\n",
			 yaxis_label) < 0)
			goto WRITE_GRAPH2D_ERROR;
		}
	if (title == NULL)
		{
		if (fprintf(tcl_input, "option add *graph.title \"%s\"\n",
			 " ") < 0)
			goto WRITE_GRAPH2D_ERROR;
		}
	else
		{
		if (fprintf(tcl_input, "option add *graph.title \"%s\"\n",
			 title) < 0)
			goto WRITE_GRAPH2D_ERROR;
		}

	k = 0;
	for (i = 0; i < nlines; i++)
		{
		  lw = lineWidth;
		  if(!strcmp(legend[i],"Model")) lw=2;
		if (fprintf(tcl_input, "set X%d { ", i+1) < 0)
			goto WRITE_GRAPH2D_ERROR;
		for (j = 0; j < npoints[i]; j++)
			{
			  if (!isnanf(y[k+j])) {
			    if (fprintf(tcl_input, "%g ", x[k+j]) < 0)
			      goto WRITE_GRAPH2D_ERROR;
			  }
			}
		if (fprintf(tcl_input, "}\n") < 0)
			goto WRITE_GRAPH2D_ERROR;

		if (fprintf(tcl_input, "set Y%d { ", i+1) < 0)
			goto WRITE_GRAPH2D_ERROR;
		for (j = 0; j < npoints[i]; j++)
			{
			  /* ALT NaN pseudo fix */
			  if (isnanf(y[k+j])) {
			    lw = 0;
			  }
			  else {
			    if (fprintf(tcl_input, "%g ", y[k+j]) < 0)
			      goto WRITE_GRAPH2D_ERROR;
			  }
			}
		if (fprintf(tcl_input, "}\n") < 0)
			goto WRITE_GRAPH2D_ERROR;
				
		if (nlines > 1) /* show the label for each line type */
		  {
		  if (fprintf(tcl_input,
		     "$graph element create \"line%d\" -xdata $X%d -ydata "
		     "$Y%d -symbol %s -bg black -foreground %s -linewidth "
		     "%d -label \"%s\"\n",
		     i+1, i+1, i+1, symbol[i], color[i], lw, 
		     legend[i]) < 0)
		     	goto WRITE_GRAPH2D_ERROR;
		  }
		else
		  { /* don't show the label for each line type, since 1 only */
		  if (fprintf(tcl_input,
		     "$graph element create \"line%d\" -xdata $X%d -ydata "
		     "$Y%d -symbol %s -bg black -foreground %s -linewidth "
		     "%d -label \" \"\n",
		     i+1, i+1, i+1, symbol[i], color[i], lw) < 0)
		     	goto WRITE_GRAPH2D_ERROR;
		  }
		k += npoints[i];
		}

	/* new addition upon Sarav's request */
	hlineval = getenv("ADD2DHLINE");
	if (hlineval != NULL) {
	  value = atof(hlineval);
	  fprintf(tcl_input, "%s\n", 
		  "scan [$graph xaxis limits] \"%s %s\" xmin xmax");

	  fprintf(tcl_input, "%s\n", 
		  "set XXX { $xmin $xmax }");
	  fprintf(tcl_input, "%s %g %g %s\n", 
		  "set YYY { ", value, value, " }");
	  fprintf(tcl_input, "%s \"%s\"\n", 
		  "$graph element create \"lineH\" -xdata $XXX -ydata $YYY -bg black -foreground red -linewidth 1 -label ", hlineval);
	}

	}
else
	goto CREATE_GRAPH2D_ERROR;

fflush(tcl_input); 
if (fclose(tcl_input) != 0)
	goto CLOSE_GRAPH2D_ERROR;

/*
  if (system(graph2d_cmd) == 0)
	status = PAVE_SUCCESS;
	*/
 putenv("BLT_GRAPH_WINDOW_ID=0"); /* reset to default non-valid window ID */
 if ((cmdpipe = popen(graph2d_cmd, "r")) != NULL) {

   int fd;
   fd = fileno(cmdpipe);
   fcntl(fd, F_SETFL, O_NONBLOCK);

   i = 5; /* make five attemtps to receive BLT_GRAPH_WINDOW_ID */
   while (i>0) {
     if (fgets(response, BUFSIZE, cmdpipe) != NULL) {
       if(!strncmp(response,"BLT_GRAPH_WINDOW_ID=",20)) {
	 putenv(response);
       }
       break;
     }
     else {
       i--;
       sleep(1);
     }
   }
   pclose(cmdpipe);
   status = PAVE_SUCCESS;
 }
if (tmpfilename != NULL)
	free(tmpfilename);
if (graph2d_cmd != NULL)
	free(graph2d_cmd);
return(status);

ALLOCATE_GRAPH2D_ERROR:
sprintf(message, "Error allocating file name.\n");
if (tmpfilename != NULL)
	free(tmpfilename);
if (graph2d_cmd != NULL)
	free(graph2d_cmd);
return(status);

CREATE_GRAPH2D_ERROR:
sprintf(message, "Error creating file: %s\n", tmpfilename);
if (tmpfilename != NULL)
	free(tmpfilename);
if (graph2d_cmd != NULL)
	free(graph2d_cmd);
return(status);

WRITE_GRAPH2D_ERROR:
sprintf(message, "Error writing to file: %s\n", tmpfilename);
fclose(tcl_input);
if (tmpfilename != NULL)
	free(tmpfilename);
if (graph2d_cmd != NULL)
	free(graph2d_cmd);
return(status);

CLOSE_GRAPH2D_ERROR:
sprintf(message, "Error closing file: %s\n", tmpfilename);
if (tmpfilename != NULL)
	free(tmpfilename);
if (graph2d_cmd != NULL)
	free(graph2d_cmd);
return(status);
}

