/****************************************************************************
 *
 * Project Title: Environmental Decision Support System
 *
 *         File: @(#)map_overlay.c	2.1
 *     Pathname: /env/proj/archive/edss/src/pave/pave_graph/SCCS/s.map_overlay.c
 * Last updated: 12/15/97 16:24:53
 *
 * Made available by MCNC and the Carolina Environmental Program of UNC Chapel
 * Hill under terms of the GNU Public License.  See gpl.txt for more details.
 *
 * See file COPYRIGHT for license information on this and supporting software.
 *
 * Carolina Environmental Program
 * University of North Carolina at Chapel Hill
 * 137 E. Franklin St.
 * Chapel Hill, NC 27599-6116
 *
 * See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
 *
 ****************************************************************************/

/****************************************************************************
	--------------------
	Modification History
	--------------------
--- ----   ----
Who When   What
--- ----   ----
*****************************************************************************/

static const char *SCCS_ID = "@(#)map_overlay.c	1.20 /tmp_mnt/pub/storage/edss/framework/src/pave/pave_graph/SCCS/s.map_overlay.c 09 Apr 1996 09:52:29";

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#ifdef _AIX
#include <fcntl.h>
#else
#include <sys/fcntl.h>
#endif /* ifdef _AIX */

#include "vis_data.h"

#define PAVE_SUCCESS 1
#define FAILURE 0

#define LAMBERT_FILENAME  "lambert/mm4d.sized"
#define LAMBERT_FILENAME2 "lambert/mm4d_nolatlon.sized"
#define UTM_CO_FILENAME   "utm/counties.arc"
#define UTM_ST_FILENAME   "utm/states.arc"

static float xminedge;
static float yminedge;
static float xmaxedge;
static float ymaxedge;

static char string[80];

int map_overlay(VIS_DATA info, float *xpts, float *ypts, int *n, 
	int *npolyline, int maxpoints, int map_options, char *message)
{

int lambert_map(float llx, float lly, float urx, float ury, int ncol, int nrow,
	int clip_xmin, int clip_xmax, int clip_ymin, int clip_ymax,
	float *xpts, float *ypts, int *n, int *npolyline, int maxpoints,
	int map_options);
int utm_map(float llx, float lly, float urx, float ury, int utm_zone,
	int ncol, int nrow,
	int clip_xmin, int clip_xmax, int clip_ymin, int clip_ymax,
	float *xpts, float *ypts, int *n, int *npolyline, int maxpoints,
	int map_options);
int latlon_map(float llx, float lly, float urx, float ury, int utm_zone,
	int ncol, int nrow,
	int clip_xmin, int clip_xmax, int clip_ymin, int clip_ymax,
	float *xpts, float *ypts, int *n, int *npolyline, int maxpoints,
	int map_options);

float llx, lly, urx, ury, dx, dy;
int status;
char tstring[256];

/* netCDF header variables for map */
int grid_type, ncol, nrow;
float xorig, yorig, xcell, ycell, xcent, ycent, p_gam, p_bet, p_alp;

/* unique variable for UTM map */
int utm_zone;

status = PAVE_SUCCESS;
switch(info.dataset)
	{
	case UAM_DATA: 
	case UAMV_DATA:
		{
		if (sscanf(info.map_info, "%g%g%g%g%d%d%d",
			&llx, &lly, &urx, &ury, &utm_zone, &ncol, &nrow) == 7)
			{ 
			/* cheesy way to figure out LatLon vs UTM */
			if((lly < 90.) && (lly>0.) && (ury<90.) && (ury>0.)) {
			  if (!latlon_map(llx, lly, urx, ury, utm_zone, 
					  ncol, nrow, 
					  info.col_min - 1,
					  ncol-info.col_max,
					  info.row_min - 1,
					  nrow-info.row_max,
					  xpts, ypts, n, npolyline,
					  maxpoints, map_options))
			    {
			      status = FAILURE;
			      sprintf(message, "Failure to load map information.");
			    }
			}
			
			else {
			llx /= 1000.0;
			lly /= 1000.0;
			urx /= 1000.0;
			ury /= 1000.0;
			if (!utm_map(llx, lly, urx, ury, utm_zone, ncol, nrow, 
				     info.col_min - 1,
				     ncol-info.col_max,
				     info.row_min - 1,
				     nrow-info.row_max,
				     xpts, ypts, n, npolyline,
				     maxpoints, map_options))
			  {
			    status = FAILURE;
			    sprintf(message, "Failure to load map information.");
			  }
		      }
		      }
		else
			{
			status = FAILURE;
			sprintf(message, "Error reading map info string");
			}
		break;
		}
	case netCDF_DATA:
		{
		if (sscanf(info.map_info, "%d%g%g%g%g%g%g%g%g%g%d%d",
			&grid_type, &xorig, &yorig, &xcell, &ycell, &xcent,
			&ycent, &p_gam, &p_bet, &p_alp, &ncol, &nrow) == 12)
			{
			if (getenv("P_ALP") != NULL) {
			   	fprintf(stderr, 
				"\nReplacing your netCDF data's P_ALP==%g\n"
				"by your environment variable's P_ALP==%g\n",
                 		p_alp, atof(getenv("P_ALP"))); 
                 		p_alp = atof(getenv("P_ALP"));
				}

			if (getenv("P_BET") != NULL) {
				fprintf(stderr, 
				"\nReplacing your netCDF data's P_BET==%g\n"
				"by your environment variable's P_BET==%g\n",
                 		p_bet, atof(getenv("P_BET")));
                 		p_bet = atof(getenv("P_BET"));
				}

			if (getenv("P_GAM") != NULL) {
				fprintf(stderr, 
				"\nReplacing your netCDF data's P_GAM==%g\n"
				"by your environment variable's P_GAM==%g\n",
                 		p_gam, atof(getenv("P_GAM")));
                 		p_gam = atof(getenv("P_GAM"));
				}

			if (getenv("XCENT") != NULL) {
				fprintf(stderr, 
				"\nReplacing your netCDF data's XCENT==%g\n"
				"by your environment variable's XCENT==%g\n",
                 		xcent, atof(getenv("XCENT")));
                 		xcent = atof(getenv("XCENT"));
				}

			if (getenv("YCENT") != NULL) {
				fprintf(stderr, 
				"\nReplacing your netCDF data's YCENT==%g\n"
				"by your environment variable's YCENT==%g\n",
                 		ycent, atof(getenv("YCENT")));
                 		ycent = atof(getenv("YCENT"));
				}

			if (getenv("XORIG") != NULL) {
				fprintf(stderr, 
				"\nReplacing your netCDF data's XORIG==%g\n"
				"by your environment variable's XORIG==%g\n",
                 		xorig, atof(getenv("XORIG")));
                 		xorig = atof(getenv("XORIG"));
				}

			if (getenv("YORIG") != NULL) {
				fprintf(stderr, 
				"\nReplacing your netCDF data's YORIG==%g\n"
				"by your environment variable's YORIG==%g\n",
                 		yorig, atof(getenv("YORIG")));
                 		yorig = atof(getenv("YORIG"));
				}

			if (getenv("XCELL") != NULL) {
				fprintf(stderr, 
				"\nReplacing your netCDF data's XCELL==%g\n"
				"by your environment variable's XCELL==%g\n",
                 		xcell, atof(getenv("XCELL")));
                 		xcell = atof(getenv("XCELL"));
				}

			if (getenv("YCELL") != NULL) {
				fprintf(stderr, 
				"\nReplacing your netCDF data's YCELL==%g\n"
				"by your environment variable's YCELL==%g\n",
                 		ycell, atof(getenv("YCELL")));
                 		ycell = atof(getenv("YCELL"));
				}


                        if (	(p_alp != 30) || 
				(p_bet != 60) || 
				(p_gam != -90)) /* HACK */
                           {
                           fprintf(stderr, 
                           "Resetting (p_alp, p_bet, p_gam) from "
                           "(%g, %g, %g) to (30, 60, -90) as per Talat Odman\n",
                           p_alp, p_bet, p_gam);
                           p_alp = 30;
                           p_bet = 60;
                           p_gam = -90;
                           }

			if ((grid_type != 2 /*1 SRT */) || 
			    (xcent != -90) || (ycent != 40) ||
			    (p_alp != 30) || (p_bet != 60) || 
			    (p_gam != -90))
				{
				/*status = FAILURE; */
				sprintf(message,
			    "Only Lambert map available is for\n"
			    "type 2=Lambert with cent (-90,40) & angle (30,60,-90),\n"
			    "and you are using\n"
			    "type %d=", grid_type);
				switch(grid_type)
				   	{
					case 1: strcat(message, "Lat/Lon"); 
						break;

					case 2: strcat(message, "Lambert"); 
						break;

					case 3: strcat(message, "Mercater"); 
                                                break;

					case 4: strcat(message, "Stereographic"); 
                                                break;

					case 5: strcat(message, "UTM"); 
                                                break;

					default: strcat(message, "Unknown Type"); 
                                                break;
					}
				sprintf(tstring,
				   " with cent (%g,%g) & angle (%g,%g,%g)\n",
                            	    xcent, ycent, p_alp, p_bet, p_gam);
				strcat(message, tstring);
				fprintf(stderr, message);
				message[0]='\0';
				fprintf(stderr, 
				"But I'll try it anyway as per Talat Odman,\n"
				"using EDSS_MAPDIR=%s\n", 
				getenv("EDSS_MAPDIR"));
				}
		/*	else  */
				{
				llx = xorig;
				lly = yorig;
				urx = xorig + xcell * ncol;
				ury = yorig + ycell * nrow;
				if (!lambert_map(llx, lly, urx, ury,
					ncol, nrow, 
					info.col_min - 1,
					ncol-info.col_max,
					info.row_min - 1,
					nrow-info.row_max,
					xpts, ypts, n, npolyline, 
					maxpoints, map_options))
					{
					status = FAILURE;
					sprintf(message, "Failure to load map information.");
					}
				}
			}
		else
			{
			status = FAILURE;
			sprintf(message, "Error reading map info string");
			}
		break;
		}
	}
return(status);
}

int lambert_map(float llx, float lly, float urx, float ury, int ncol, int nrow,
	int clip_xmin, int clip_xmax, int clip_ymin, int clip_ymax,
	float *xpts, float *ypts, int *n, int *npolyline, 
	int maxpoints, int map_options)
{
register int i, j;
int segment;
float xmin;
float ymin;
float xmax;
float ymax;
float xcellsize;
float ycellsize;
float x0;
float y0;
float x1;
float y1;
float x;
float y;
float tempx0;
float tempy0;
float tempx1;
float tempy1;
float lastx;
float lasty;
float xscale;
float yscale;
int nx;
int ny;
int nxbase;
int nybase;
FILE *map_file;
int npoints;
int npoly;
int used;
int total;
float clipx0;
float clipy0;
float clipx1;
float clipy1;

char *cell_base = "cell edge";

char *lambert_filename = NULL;

xmin = llx;
ymin = lly;
xmax = urx;
ymax = ury;
nx = ncol;
ny = nrow;
	
xcellsize = (xmax - xmin)/((float) nx);
ycellsize = (ymax - ymin)/((float) ny);

if (!strcmp(cell_base, "cell center"))	/* field to mesh types */
	{
	nxbase = nx - 1;
	nybase = ny - 1;
	}
else if (!strcmp(cell_base, "cell edge"))	/* tile types */
	{
	nxbase = nx;
	nybase = ny;
	}
xscale = ((float) (nxbase))/(xmax - xmin);
yscale = ((float) (nybase))/(ymax - ymin);

if ((lambert_filename = ((char *) malloc(sizeof(char) * 512))) == NULL)
	{
	fprintf(stderr, "Can't allocate memory for utm file name\n");
	return(FAILURE);
	}

if (map_options & MAP_NO_LATLON)
	sprintf(lambert_filename, "%s/%s", 
		getenv("EDSS_MAPDIR"), LAMBERT_FILENAME2);
else
	sprintf(lambert_filename, "%s/%s",
		getenv("EDSS_MAPDIR"), LAMBERT_FILENAME);

if ((map_file = fopen(lambert_filename, "r")) == NULL)
	{
	fprintf(stderr, "Cannot open map file %s\n", LAMBERT_FILENAME2);
	free(lambert_filename);
	return(FAILURE);
	}

clipx0 = (float) clip_xmin;
clipy0 = (float) clip_ymin;
clipx1 = (float) (nxbase) - clip_xmax;
clipy1 = (float) (nybase) - clip_ymax;

npoly = 0;
used = 0;
total = 0;
while (NULL != fgets(string, 79, map_file))
	{
	sscanf(string, "%d", &segment);
	fgets(string, 79, map_file);

	npoints = 0;
	while (!((string[0] == 'E') && (string[1] == 'N') &&
		(string[2] == 'D')))
		{
		sscanf(string, "%g%g", &x, &y);         
		tempx0 = (x - xmin) * xscale;
		tempy0 = (y - ymin) * yscale;
		if (npoints > 0)
			{
			x0 = tempx1;
			y0 = tempy1;
			x1 = tempx0;
			y1 = tempy0;
			if (clip(&x0, &y0, &x1, &y1,
				clipx0, clipy0, clipx1, clipy1))
				{
				if (!used)
					{
					if (used+total+2 < maxpoints)
						{
						xpts[used+total] = x0;
						ypts[used+total] = y0;
						xpts[used+1+total] = x1;
						ypts[used+1+total] = y1;
						used += 2;
						}
					}
				else
					{
					if ((x0 == lastx) && (y0 == lasty))
						{
						if (used+total+1 < maxpoints)
							{
							xpts[used+total] = x1;
							ypts[used+total] = y1;
							++used;
							}
						}
					else
						{
						n[npoly] = used;
						++npoly;
						total += used;
						used = 0;
						if (used+total+2 < maxpoints)
							{
							xpts[used+total] = x0;
							ypts[used+total] = y0;
							xpts[used+1+total] = x1;
							ypts[used+1+total] = y1;
							used += 2;
							}
						}
					}
				lastx = x1;
				lasty = y1;
				}
			}
		tempx1 = tempx0;
		tempy1 = tempy0;
		++npoints;
		fgets(string, 79, map_file);
		}
	if (used)
		{
		n[npoly] = used;
		++npoly;
		total += used;
		used = 0;
		}
	}
fclose(map_file);
free(lambert_filename);

if (used+total+5 < maxpoints)
	{
	xpts[used+total] = clipx0; ypts[used+total] = clipy0;
	++used;
	xpts[used+total] = clipx1; ypts[used+total] = clipy0;
	++used;
	xpts[used+total] = clipx1; ypts[used+total] = clipy1;
	++used;
	xpts[used+total] = clipx0; ypts[used+total] = clipy1;
	++used;
	xpts[used+total] = clipx0; ypts[used+total] = clipy0;
	++used;
	}

if (used)
	{
	n[npoly] = used;
	++npoly;
	total += used;
	}
*npolyline = npoly;
return(PAVE_SUCCESS);

}


int utm_map(float llx, float lly, float urx, float ury, int utm_zone,
	int ncol, int nrow,
	int clip_xmin, int clip_xmax, int clip_ymin, int clip_ymax,
	float *xpts, float *ypts, int *n, int *npolyline, int maxpoints,
	int map_options)
{
int set_utm_position(int map_fd, int index_fd, int *npoints);
void geo2utm(float *utmx, float *utmy, int utm_zone, float lon, float lat);

register int i, j;
static float point[2];
int segment;
float xmin;
float ymin;
float xmax;
float ymax;
float xcellsize;
float ycellsize;
float x0;
float y0;
float x1;
float y1;
float x;
float y;
float tempx0;
float tempy0;
float tempx1;
float tempy1;
float lastx;
float lasty;
float xscale;
float yscale;
int nx;
int ny;
int nxbase;
int nybase;
int map_fd;
int index_fd;
int npoints;
int npoly;
int used;
int total;
float clipx0;
float clipy0;
float clipx1;
float clipy1;

char *cell_base = "cell edge";

char *utm_filename = NULL;
char *utm_indexname = NULL;

xmin = llx;
ymin = lly;
xmax = urx;
ymax = ury;
nx = ncol;
ny = nrow;
	
xcellsize = (xmax - xmin)/((float) nx);
ycellsize = (ymax - ymin)/((float) ny);


if (!strcmp(cell_base, "cell center"))	/* field to mesh types */
	{
	nxbase = nx - 1;
	nybase = ny - 1;
	}
else if (!strcmp(cell_base, "cell edge"))	/* tile types */
	{
	nxbase = nx;
	nybase = ny;
	}
xscale = ((float) (nxbase))/(xmax - xmin);
yscale = ((float) (nybase))/(ymax - ymin);

if ((utm_filename = ((char *) malloc(sizeof(char) * 512))) == NULL)
	{
	fprintf(stderr, "Can't allocate memory for utm file name\n");
	return(FAILURE);
	}
if ((utm_indexname = ((char *) malloc(sizeof(char) * 512))) == NULL)
	{
	fprintf(stderr, "Can't allocate memory for utm index file name\n");
	free(utm_filename);
	return(FAILURE);
	}

if (map_options & MAP_UTM_STATES)
	sprintf(utm_filename, "%s/%s", getenv("EDSS_MAPDIR"), UTM_ST_FILENAME);
else
	sprintf(utm_filename, "%s/%s", getenv("EDSS_MAPDIR"), UTM_CO_FILENAME);

sprintf(utm_indexname, "%s.index%d", utm_filename, utm_zone);

if ((map_fd = open(utm_filename, O_RDONLY)) == -1)
	{
	fprintf(stderr, "Cannot open UTM map file %s\n", utm_filename);
	free(utm_filename); free(utm_indexname);
	return(FAILURE);
	}
if ((index_fd = open(utm_indexname, O_RDONLY)) == -1)
	{
	fprintf(stderr, "Cannot open UTM index file %s\n", utm_indexname);
	fprintf(stderr, "See instructions for creating UTM index files.\n");
	close(map_fd);
	free(utm_filename); free(utm_indexname);
	return(FAILURE);
	}

clipx0 = (float) clip_xmin;
clipy0 = (float) clip_ymin;
clipx1 = (float) (nxbase) - clip_xmax;
clipy1 = (float) (nybase) - clip_ymax;

xminedge = xmin + xcellsize * clip_xmin;
yminedge = ymin + ycellsize * clip_ymin;
xmaxedge = xmax - xcellsize * clip_xmax;
ymaxedge = ymax - ycellsize * clip_ymax;

npoly = 0;
used = 0;
total = 0;

while (set_utm_position(map_fd, index_fd, &npoints))
	{
	for (i = 0; i < npoints; ++i)
		{
		if (read(map_fd, (char *) point, 8) != 8)
			{
			close(map_fd); close(index_fd);
			free(utm_filename); free(utm_indexname);
			return(FAILURE);
			}
		point[0] = (float) fabs((double) point[0]);
		geo2utm(&x, &y, utm_zone, point[0], point[1]);

		tempx0 = (x - xmin) * xscale;
		tempy0 = (y - ymin) * yscale;
		if (i > 0)
			{
			x0 = tempx1;
			y0 = tempy1;
			x1 = tempx0;
			y1 = tempy0;
			if (clip(&x0, &y0, &x1, &y1,
				clipx0, clipy0, clipx1, clipy1))
				{
				if (!used)
					{
					if (used+total+2 < maxpoints)
						{
						xpts[used+total] = x0;
						ypts[used+total] = y0;
						xpts[used+1+total] = x1;
						ypts[used+1+total] = y1;
						used += 2;
						}
					}
				else
					{
					if ((x0 == lastx) && (y0 == lasty))
						{
						if (used+total+2 < maxpoints)
							{
							xpts[used+total] = x1;
							ypts[used+total] = y1;
							++used;
							}
						}
					else
						{
						n[npoly] = used;
						++npoly;
						total += used;
						used = 0;
						if (used+total+2 < maxpoints)
							{
							xpts[used+total] = x0;
							ypts[used+total] = y0;
							xpts[used+1+total] = x1;
							ypts[used+1+total] = y1;
							used += 2;
							}
						}
					}
				lastx = x1;
				lasty = y1;
				}
			}
		tempx1 = tempx0;
		tempy1 = tempy0;
		}
	if (used)
		{
		n[npoly] = used;
		++npoly;
		total += used;
		used = 0;
		}
	}
close(map_fd); close(index_fd);
free(utm_filename); free(utm_indexname);

if (used+total+5 < maxpoints)
	{
	xpts[used+total] = clipx0; ypts[used+total] = clipy0;
	++used;
	xpts[used+total] = clipx1; ypts[used+total] = clipy0;
	++used;
	xpts[used+total] = clipx1; ypts[used+total] = clipy1;
	++used;
	xpts[used+total] = clipx0; ypts[used+total] = clipy1;
	++used;
	xpts[used+total] = clipx0; ypts[used+total] = clipy0;
	++used;
	}	

if (used)
	{
	n[npoly] = used;
	++npoly;
	total += used;
	}

/*
printf("Total points = %d\n", total);
*/
*npolyline = npoly;
return(PAVE_SUCCESS);
}

int set_utm_position(int map_fd, int index_fd, int *npoints)
{
static float corners[4];
static off_t position;
int inside;

while (read(index_fd, (char *) corners, 16) == 16)
	{
	inside = 0;

/* check for cases where one corner is inside the clip box */

	if (
		(
		((corners[0] >= xminedge) && (corners[0] <= xmaxedge))||
		((corners[2] >= xminedge) && (corners[2] <= xmaxedge)) 
		) && 
		(
		((corners[1] >= yminedge) && (corners[1] <= ymaxedge))||
		((corners[3] >= yminedge) && (corners[3] <= ymaxedge)) 
		)
	    )
		inside = 1;

/* check for other cases where x or y coordinates span the entire clip box */

	if (!inside)
		{
		if (
			(
		((corners[0] <= xminedge) && (corners[2] >= xmaxedge))&&
		((corners[1] >= yminedge) || (corners[3] <= ymaxedge)) 
			) || 
			(
		((corners[1] <= yminedge) && (corners[3] >= ymaxedge))&&
		((corners[0] >= xminedge) || (corners[2] <= xmaxedge)) 
			)
		   )
			inside = 1;
		}

	if (inside)
		{
		if (read(index_fd, (char *) (&position), 4) != 4)
			{
			close(map_fd); close(index_fd);
			return(FAILURE);
			}

		/* skip to arc node position and 6th element in the
		   node header which contains npoints */
		if (lseek(map_fd, (off_t) (position + 20),
			SEEK_SET) == -1)
			{
			close(map_fd); close(index_fd);
			return(FAILURE);
			}
		if (read(map_fd, (char *) (npoints), 4) != 4)
			{
			close(map_fd); close(index_fd);
			return(FAILURE);
			}
		}
	else
		{
		/* skip position index information */
		if (lseek(index_fd, (off_t) 4, SEEK_CUR) == -1)
			{
			close(map_fd); close(index_fd);
			return(FAILURE);
			}
		}
	if (inside)
		{
		return(PAVE_SUCCESS);
		}
	}
return(FAILURE);
}

/***********************************************************************/
/*   The routine below is from William Ivey's "project.c" functions    */
/***********************************************************************/
/* geo2utm: convert coordinates from geographic decimal degrees to utm */
/***********************************************************************/

void geo2utm(float *utmx, float *utmy, int utm_zone, float lon, float lat)
{
#define degrad 0.017453

float utmym, dlong;

utmym = 2.41 + 110.268 * lat + 0.00903 * (lat*lat);
dlong = 180. - (6 * utm_zone) + (3 - lon);
*utmy = 3187 * sin(degrad * 2 * lat) * (1 - cos(degrad * dlong)) + utmym;
*utmx = (111.226 + 0.0053 * lat) * cos(degrad * lat) * dlong + 500.;
}

int latlon_map(float llx, float lly, float urx, float ury, int utm_zone,
	int ncol, int nrow,
	int clip_xmin, int clip_xmax, int clip_ymin, int clip_ymax,
	float *xpts, float *ypts, int *n, int *npolyline, int maxpoints,
	int map_options)
{
int set_utm_position(int map_fd, int index_fd, int *npoints);

register int i, j;
static float point[2];
int segment;
float xmin;
float ymin;
float xmax;
float ymax;
float xcellsize;
float ycellsize;
float x0;
float y0;
float x1;
float y1;
float x;
float y;
float tempx0;
float tempy0;
float tempx1;
float tempy1;
float lastx;
float lasty;
float xscale;
float yscale;
int nx;
int ny;
int nxbase;
int nybase;
int map_fd;
int index_fd;
int npoints;
int npoly;
int used;
int total;
float clipx0;
float clipy0;
float clipx1;
float clipy1;

char *cell_base = "cell edge";

char *utm_filename = NULL;
char *utm_indexname = NULL;

xmin = llx;
ymin = lly;
xmax = urx;
ymax = ury;
nx = ncol;
ny = nrow;
	
xcellsize = (xmax - xmin)/((float) nx);
ycellsize = (ymax - ymin)/((float) ny);


if (!strcmp(cell_base, "cell center"))	/* field to mesh types */
	{
	nxbase = nx - 1;
	nybase = ny - 1;
	}
else if (!strcmp(cell_base, "cell edge"))	/* tile types */
	{
	nxbase = nx;
	nybase = ny;
	}
xscale = ((float) (nxbase))/(xmax - xmin);
yscale = ((float) (nybase))/(ymax - ymin);

if ((utm_filename = ((char *) malloc(sizeof(char) * 512))) == NULL)
	{
	fprintf(stderr, "Can't allocate memory for utm file name\n");
	return(FAILURE);
	}
if ((utm_indexname = ((char *) malloc(sizeof(char) * 512))) == NULL)
	{
	fprintf(stderr, "Can't allocate memory for utm index file name\n");
	free(utm_filename);
	return(FAILURE);
	}

/*
if (fabs(xmax-xmin)<5.0 || fabs(ymax-ymin)<5.0) {
  sprintf(utm_filename, "%s/%s", getenv("EDSS_MAPDIR"), UTM_CO_FILENAME);
}
else {
  sprintf(utm_filename, "%s/%s", getenv("EDSS_MAPDIR"), UTM_ST_FILENAME);
}
*/

sprintf(utm_filename, "%s/%s", getenv("EDSS_MAPDIR"), UTM_ST_FILENAME);
sprintf(utm_indexname, "%s.latlon.index", utm_filename);

if ((map_fd = open(utm_filename, O_RDONLY)) == -1)
	{
	fprintf(stderr, "Cannot open UTM map file %s\n", utm_filename);
	free(utm_filename); free(utm_indexname);
	return(FAILURE);
	}
if ((index_fd = open(utm_indexname, O_RDONLY)) == -1)
	{
	fprintf(stderr, "Cannot open UTM index file %s\n", utm_indexname);
	fprintf(stderr, "See instructions for creating UTM index files.\n");
	close(map_fd);
	free(utm_filename); free(utm_indexname);
	return(FAILURE);
	}

clipx0 = (float) clip_xmin;
clipy0 = (float) clip_ymin;
clipx1 = (float) (nxbase) - clip_xmax;
clipy1 = (float) (nybase) - clip_ymax;

xminedge = xmin + xcellsize * clip_xmin;
yminedge = ymin + ycellsize * clip_ymin;
xmaxedge = xmax - xcellsize * clip_xmax;
ymaxedge = ymax - ycellsize * clip_ymax;

npoly = 0;
used = 0;
total = 0;

while (set_utm_position(map_fd, index_fd, &npoints))
	{
	for (i = 0; i < npoints; ++i)
		{
		if (read(map_fd, (char *) point, 8) != 8)
			{
			close(map_fd); close(index_fd);
			free(utm_filename); free(utm_indexname);
			return(FAILURE);
			}

		x=point[0];
		y=point[1];

		tempx0 = (x - xmin) * xscale;
		tempy0 = (y - ymin) * yscale;
		if (i > 0)
			{
			x0 = tempx1;
			y0 = tempy1;
			x1 = tempx0;
			y1 = tempy0;
			if (clip(&x0, &y0, &x1, &y1,
				clipx0, clipy0, clipx1, clipy1))
				{
				if (!used)
					{
					if (used+total+2 < maxpoints)
						{
						xpts[used+total] = x0;
						ypts[used+total] = y0;
						xpts[used+1+total] = x1;
						ypts[used+1+total] = y1;
						used += 2;
						}
					}
				else
					{
					if ((x0 == lastx) && (y0 == lasty))
						{
						if (used+total+2 < maxpoints)
							{
							xpts[used+total] = x1;
							ypts[used+total] = y1;
							++used;
							}
						}
					else
						{
						n[npoly] = used;
						++npoly;
						total += used;
						used = 0;
						if (used+total+2 < maxpoints)
							{
							xpts[used+total] = x0;
							ypts[used+total] = y0;
							xpts[used+1+total] = x1;
							ypts[used+1+total] = y1;
							used += 2;
							}
						}
					}
				lastx = x1;
				lasty = y1;
				}
			}
		tempx1 = tempx0;
		tempy1 = tempy0;
		}
	if (used)
		{
		n[npoly] = used;
		++npoly;
		total += used;
		used = 0;
		}
	}
close(map_fd); close(index_fd);
free(utm_filename); free(utm_indexname);

if (used+total+5 < maxpoints)
	{
	xpts[used+total] = clipx0; ypts[used+total] = clipy0;
	++used;
	xpts[used+total] = clipx1; ypts[used+total] = clipy0;
	++used;
	xpts[used+total] = clipx1; ypts[used+total] = clipy1;
	++used;
	xpts[used+total] = clipx0; ypts[used+total] = clipy1;
	++used;
	xpts[used+total] = clipx0; ypts[used+total] = clipy0;
	++used;
	}	

if (used)
	{
	n[npoly] = used;
	++npoly;
	total += used;
	}

/*
printf("Total points = %d\n", total);
*/
*npolyline = npoly;
return(PAVE_SUCCESS);
}
