/****************************************************************************
 *
 * Project Title: Environmental Decision Support System
 *
 *         File: @(#)ColorBox.cc	2.1
 *     Pathname: /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.ColorBox.cc
 * Last updated: 12/15/97 16:32:06
 *
 * Made available by MCNC and the Carolina Environmental Program of UNC Chapel
 * Hill under terms of the GNU Public License.  See gpl.txt for more details.
 *
 * See file COPYRIGHT for license information on this and supporting software.
 *
 * Carolina Environmental Program
 * University of North Carolina at Chapel Hill
 * 137 E. Franklin St.
 * Chapel Hill, NC 27599-6116
 *
 * See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
 *
 ****************************************************************************/

static const char *SCCS_ID = "@(#)ColorBox.cc	2.1 /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.ColorBox.cc 12/15/97 16:32:06";

/**************************************************
   color_box.c using OSF MOTIF 
   Kah Eng Pua	919-942-3756
   Copyright (C)
   Aug 18, 1994
***************************************************/

#include "ColorBox.h"

#define MAXCOLORS	256

Widget		selected_color;
xpColorData	*tmp_color_data;

/*-----------------------------------------*/
int XpGetNamedPixel(w, colorname)
   Widget w;
   char   *colorname;
{
   XColor	color, skip;
   Display	*dpy  = XtDisplay(w);
   int		scr  = DefaultScreen(dpy);
   Colormap	cmap = DefaultColormap(dpy, scr);
 
   if (XAllocNamedColor(dpy, cmap, colorname, &color, &skip)) 
      return (color.pixel);
   else {
      fprintf(stderr, "Can't allocate color %s\n",colorname);  
      return (BlackPixel(dpy, scr));
   }  
}

void XpCloseDialogCB(w,client_data, cbs)
   Widget		w;
   XtPointer		client_data;
   XmAnyCallbackStruct	*cbs;
{
   XtUnmanageChild(XtParent(w));
}


/***************************************/

static void set_current_colorCB(w, tag,reason)
   Widget 		w;
   int			tag;
   XmAnyCallbackStruct	*reason;
{

  XtVaSetValues(selected_color,
		XmNbackground, XpGetNamedPixel(w, xpRGB[tag].name),
		NULL);


   strcpy(tmp_color_data->name, xpRGB[tag].name);

   tmp_color_data->r = xpRGB[tag].r;
   tmp_color_data->g = xpRGB[tag].g;
   tmp_color_data->b = xpRGB[tag].b;

}

void create_color_bar(parent, ncolors)
   Widget	parent;
   int		ncolors;
{
   Widget	radiobox;
   int		i;

   radiobox = XmCreateRadioBox(parent, "rbox", NULL, 0);
   XtVaSetValues(radiobox, 
		XmNorientation,		XmHORIZONTAL,
		XmNnumColumns,		7,
		XmNtopAttachment,	XmATTACH_FORM,
		XmNleftAttachment,	XmATTACH_FORM,
		XmNrightAttachment,	XmATTACH_FORM,
		XmNbottomAttachment,	XmATTACH_FORM,
		NULL);

   for ( i = 0; i < ncolors; i++ )  {
        Widget toggle;

        toggle   = 
            XtVaCreateManagedWidget ( "", 
                                  xmToggleButtonWidgetClass, radiobox, 
                                  XmNbackground, XpGetNamedPixel(parent, xpRGB[i].name),
                                  NULL );
	XtAddCallback(toggle, XmNvalueChangedCallback, set_current_colorCB, i);
   }

   if (XtIsManaged(radiobox)) XtUnmanageChild(radiobox); XtManageChild(radiobox);
}


void createColorBox(parent, name, apply_cb, apply_data, color_data)
   Widget	parent;
   char		*name;
   void		(*apply_cb)();
   XtPointer	apply_data;
   xpColorData	*color_data;
{
   Widget	sw, select_color_label, appl, clos, label, sep;
   int		height = 40;
   int		num_colors;
   XColor	Colors[MAXCOLORS];
   int		i;

   tmp_color_data = color_data;

   color_data->wid = XmCreateFormDialog(parent, "Color Box", NULL, 0);
   XtVaSetValues(color_data->wid, 
		XmNautoUnmanage,	False,
		XmNwidth,		390,
		XmNheight,		/*220,*/ 480,
		NULL);

   label = XtVaCreateManagedWidget(name, 
		xmLabelWidgetClass,	color_data->wid,
		XmNtopAttachment,	XmATTACH_FORM,
		XmNtopOffset,		20,
		XmNleftAttachment,	XmATTACH_FORM,
		XmNleftOffset,		20,
		NULL);

   appl = XtVaCreateManagedWidget("Apply", 
		xmPushButtonWidgetClass,	color_data->wid,
		XmNwidth,		240,
		XmNleftAttachment,	XmATTACH_FORM,
		XmNbottomAttachment,	XmATTACH_FORM,
		XmNheight,		40,
		NULL);
   XtAddCallback(appl, XmNactivateCallback,  apply_cb, apply_data);

   clos = XtVaCreateManagedWidget("Close", 
		xmPushButtonWidgetClass,	color_data->wid,
		XmNleftAttachment,	XmATTACH_WIDGET,
		XmNleftWidget,		appl,
		XmNrightAttachment,	XmATTACH_FORM,
		XmNbottomAttachment,	XmATTACH_FORM,
		XmNheight,		40,
		NULL);
   XtAddCallback(clos, XmNactivateCallback,  XpCloseDialogCB, clos);

   sep = XtVaCreateManagedWidget("", 
		xmSeparatorWidgetClass,	color_data->wid,
		XmNbottomAttachment,	XmATTACH_WIDGET,
		XmNbottomWidget,	clos,
		XmNbottomOffset,	20,
		XmNleftAttachment,	XmATTACH_FORM,
		XmNrightAttachment,	XmATTACH_FORM,
		NULL);

   select_color_label = XtVaCreateManagedWidget("Selected Color", 
		xmLabelWidgetClass,	color_data->wid,
		XmNbottomAttachment,	XmATTACH_WIDGET,
		XmNbottomWidget,	sep,
		XmNbottomOffset,	30,
		XmNleftAttachment,	XmATTACH_FORM,
		XmNleftOffset,		30,
		NULL);

   selected_color = XtVaCreateManagedWidget("", 
		xmPushButtonWidgetClass,	color_data->wid,
		XmNrightAttachment,	XmATTACH_FORM,
		XmNrightOffset,		20,
		XmNleftAttachment,	XmATTACH_WIDGET,
		XmNleftWidget,		select_color_label,
		XmNleftOffset,		20,
		XmNbottomAttachment,	XmATTACH_WIDGET,
		XmNbottomWidget,	sep,
		XmNbottomOffset,	20,
		XmNwidth,		70,
		XmNheight,		70,
		XmNbackground,		XpGetNamedPixel(color_data->wid, "red"),
		NULL);
   strcpy(color_data->name, "red");

   sw = XtVaCreateManagedWidget("frame",
		xmScrolledWindowWidgetClass,	color_data->wid,
		XmNscrollBarDisplayPolicy,	XmAS_NEEDED,
		XmNscrollingPolicy,		XmAUTOMATIC,
		XmNtopAttachment,		XmATTACH_WIDGET,
		XmNtopWidget,			label,
		XmNtopOffset,			20,
		XmNleftAttachment,		XmATTACH_FORM,
		XmNleftOffset,			20,
		XmNrightAttachment,		XmATTACH_FORM,
		XmNrightOffset,			20,
		XmNbottomAttachment,		XmATTACH_WIDGET,
		XmNbottomWidget,		selected_color,
		XmNbottomOffset,		20,
		NULL);

   create_color_bar(sw, num_rgb_names);

}

