/****************************************************************************
 *
 * Project Title: Environmental Decision Support System
 *
 *         File: @(#)DrawScale.cc	2.1
 *     Pathname: /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.DrawScale.cc
 * Last updated: 12/15/97 16:32:43
 *
 * Made available by MCNC and the Carolina Environmental Program of UNC Chapel
 * Hill under terms of the GNU Public License.  See gpl.txt for more details.
 *
 * See file COPYRIGHT for license information on this and supporting software.
 *
 * Carolina Environmental Program
 * University of North Carolina at Chapel Hill
 * 137 E. Franklin St.
 * Chapel Hill, NC 27599-6116
 *
 * See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
 *
 ****************************************************************************/

static const char *SCCS_ID = "@(#)DrawScale.cc	2.1 /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.DrawScale.cc 12/15/97 16:32:43";

//////////////////////////////////////////////////////////////////////
//	Drawscale.C
//	K. Eng Pua
//	May 12, 1993
//	
//////////////////////////////////////////////////////////////////////
#include "DrawScale.h"

void DrawScale::scaleInit(float x1, float y1, float x2, float y2, 
			int wmaxx, int wmaxy,
			int offset_left, int offset_right,
			int offset_top,	 int offset_bottom)
{
   xmin_ = x1;
   xmax_ = x2;
   ymin_ = y1;
   ymax_ = y2;
   offset_left_   = offset_left;
   offset_right_  = offset_right;
   offset_top_    = offset_top;
   offset_bottom_ = offset_bottom;

   sx_   = ((float) (wmaxx-offset_left_-offset_right_)) / 
		(float)(xmax_-xmin_);
   sy_   = ((float) (wmaxy-offset_top_-offset_bottom_)) / 
		(float)(ymax_-ymin_);


#ifdef DIAGNOSTICS
   fprintf(stderr, "xmin=%f\n",xmin_);
   fprintf(stderr, "xmax=%f\n",xmax_);
   fprintf(stderr, "ymin=%f\n",ymin_);
   fprintf(stderr, "ymax=%f\n",ymax_);
   fprintf(stderr, "wmaxx=%d\n",wmaxx);
   fprintf(stderr, "wmaxy=%d\n",wmaxy);
   fprintf(stderr, "sx=%f\n",sx_);
   fprintf(stderr, "sy=%f\n",sy_);
#endif
}


int DrawScale::scalex(float x)
{
   return (int) ((x-xmin_)*sx_) + offset_left_;
}


int DrawScale::scaley(float y)
{
   return (int) ((ymax_-y)*sy_) + offset_top_;
}



float DrawScale::fscalex(float x)
{
   return (float) ((x-xmin_)*sx_) + offset_left_;
}


float DrawScale::fscaley(float y)
{
   return (float) ((ymax_-y)*sy_) + offset_top_;
}


float DrawScale::gridx(int x)
{
  return xmin_+(float)(x - offset_left_)/sx_;
}


float DrawScale::gridy(int y)
{
  return ymax_ - (float)(y - offset_top_)/sy_;
}



