/////////////////////////////////////////////////////////////
//
// Project Title: Environmental Decision Support System
//         File: @(#)ExportServer.cc	2.1
//     Pathname: /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.ExportServer.cc
// Last updated: 12/15/97 16:32:50
//
// Made available by MCNC and the Carolina Environmental Program of UNC Chapel
// Hill under terms of the GNU Public License.  See gpl.txt for more details.
//
// See file COPYRIGHT for license information on this and supporting software.
//
// Carolina Environmental Program
// University of North Carolina at Chapel Hill
// 137 E. Franklin St.
// Chapel Hill, NC 27599-6116
//
// See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
//
/////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// --------------------
// Modification History
// --------------------
//
// ---  ----    ----
// Who  When    What
// ---  ----    ----
// SRT  950615  Implemented
// SRT  950908  added int removeAllItems(Widget dialog)
// SRT  960517  Added hooks to netCDF exporting
// SRT  961011  Added release_edata(), save_cancelCB(), and save_cancel_cb()
//
//////////////////////////////////////////////////////////////////////////////


static const char *SCCS_ID = "@(#)ExportServer.cc	2.1 /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.ExportServer.cc 12/15/97 16:32:50";

/////////////////////////////////////////////////////////////
//
// ExportServer.C
// Steve Thorpe
// thorpe@ncsc.org
// NCSC Environmental Programs, MCNC
// June 15, 1995
//
/////////////////////////////////////////////////////////////
//
// ExportServer Class 
//
/////////////////////////////////////////////////////////////


#include "ExportServer.h"

                                                // Constructor
ExportServer::ExportServer(int etype,           // type of data
                        char *name,             // name
                        Widget parent,          // parent to attach UI to
                        char *dialogtitle):     // title
	      SelectLoadSaveServer(
			name, 
			parent, 
			dialogtitle, 
			"dummyFileMarker")	
{
exportType_ = etype;
edata_ = (void *) NULL;
}


ExportServer::~ExportServer()			// destructor
{
release_edata();
}



void ExportServer::ShowUI(void *edata)         // data ptr
{
release_edata();
edata_ = edata;
if (!edata_)
	{
	Message error(parent_, XmDIALOG_ERROR, "No data passed to ExportServer::ShowUI()!");
	return;
	}
postSaveSelectionDialog();
}



void ExportServer::save_okCB(Widget, XtPointer clientData, XtPointer callData)   // override SelectLoadSaveServer's
{
   ExportServer *obj = (ExportServer *) clientData;
   XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *) callData;

   obj->save_ok_cb(cbs->value);
}


void ExportServer::save_ok_cb(XmString xmstr)              // override SelectLoadSaveServer's
{
   char *str;

   if (XmStringGetLtoR(xmstr, XmSTRING_DEFAULT_CHARSET, &str)) 
        saveSelectionListToFile(str, (Widget)NULL);

   XtFree(str);
}



void ExportServer::saveSelectionListToFile              // override SelectLoadSaveServer's
         (char *filename, Widget)       		// saveSelectionListToFile()
{
char estring[256];

#ifdef DIAGNOSTICS
fprintf(stderr, "ExportServer::saveSelectionListToFile saving to \n");
fprintf(stderr, "%s\n", filename);
#endif // DIAGNOSTICS

switch (exportType_)
	{
	case PAVE_EXPORT_AVS:	  if (dump_VIS_DATA_to_AVS_file((VIS_DATA *)edata_, 
								filename, 
								estring))
					{
					Message error(parent_, XmDIALOG_ERROR, estring);
					}
				  break;

	case PAVE_EXPORT_TABBED:  if (dump_VIS_DATA_to_tabbed_ascii_file((VIS_DATA *)edata_,
								filename, 
								estring))
					{
					Message error(parent_, XmDIALOG_ERROR, estring);
					}
				  break;

	case PAVE_EXPORT_NETCDF:  if (dump_VIS_DATA_to_netCDF_file((VIS_DATA *)edata_,
								filename, 
								estring))
					{
					Message error(parent_, XmDIALOG_ERROR, estring);
					}

				  break;

	default:		{
				Message error(parent_, XmDIALOG_ERROR, 
				"Unknown export data typE in ExportServer::saveSelectionListToFile");
				}
	}
release_edata();
}


	// override SelectionServer's
Widget ExportServer::createEditSelectionDialog(void) 
{
return (Widget) NULL;
}


	// override SelectionServer's
int ExportServer::addItem(char *, Widget) 
{
return 0;
} 


	// override SelectionServer's
int ExportServer::removeAllItems(Widget)
{
   return 0;
}


void ExportServer::release_edata(void)	// frees up the data at *edata_;
{
if (edata_ == (void *) NULL)
	return;

free_vis((VIS_DATA *)edata_); free(edata_); edata_ = (void *) NULL;
}


void ExportServer::save_cancel_cb(XmString /*xmstr*/)              // override SelectLoadSaveServer's
{
release_edata();
}
