/////////////////////////////////////////////////////////////
//
// Project Title: Environmental Decision Support System
//         File: @(#)Level.cc	2.1
//     Pathname: /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.Level.cc
// Last updated: 12/15/97 16:33:05
//
// Made available by MCNC and the Carolina Environmental Program of UNC Chapel
// Hill under terms of the GNU Public License.  See gpl.txt for more details.
//
// See file COPYRIGHT for license information on this and supporting software.
//
// Carolina Environmental Program
// University of North Carolina at Chapel Hill
// 137 E. Franklin St.
// Chapel Hill, NC 27599-6116
//
// See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
//
/////////////////////////////////////////////////////////////

static const char *SCCS_ID = "@(#)Level.cc	2.1 /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.Level.cc 12/15/97 16:33:05";

/////////////////////////////////////////////////////////////
//
// Level.C
// Steve Thorpe
// thorpe@ncsc.org
// NCSC Environmental Programs, MCNC
// May 29, 1995
//
/////////////////////////////////////////////////////////////
//
//   Every Level Object:
//
// o When requested, must provide a UI which manages a 
//   dynamic "mask" of its currently selected levels
//
// o Must notify each of the Formulas when its currently
//   selected levels change
//
// o Must retrieve and supply its data when requested
//
/////////////////////////////////////////////////////////////
//
// Revision History
// SRT	950529	Implemented
//
/////////////////////////////////////////////////////////////


#include "Level.h"

					// Constructor

Level::Level(linkedList *formulaList,   // points to the list of
					// Formula objects this
					// Object may need to 
					// interact with

	      int nlevels,     		// the number of layers this
					// object needs to manage

	      Widget parent,		// to base position of UI on

	      char *estring)
{
assert(estring != NULL);
estring[0] = '\0';
KMAX_ = nlevels;
formulaList_ = formulaList;
parent_ = parent;
levelDialogBox_ = NULL;
minLevel_ = maxLevel_ = 1;

#ifdef DIAGNOSTICS
fprintf(stderr, "Enter Level::Level(%d)\n", nlevels);
#endif // DIAGNOSTICS

if (nlevels <= 0)
	sprintf(estring, "Bad nlevels == %d", nlevels);
else 
if (nlevels > 1)
	{
	char title[50];
	char minLabel[50];
	char maxLabel[50];
	void (*valsModifiedParentCB)(void *);

	sprintf(title, "%d Layer Models", KMAX_);
	sprintf(minLabel, "Lowest Layer Of Interest");
	sprintf(maxLabel, "Highest Layer Of Interest");
	valsModifiedParentCB = (void (*)(void *)) modifiedTheseLevelsCB;

	levelDialogBox_ = new StepUI(
                title,                          // title of dialog box
                parent_,                        // to base position on
                minLabel,                       // "Step Min", "Layer Min", etc
                maxLabel,                       // "Step Max", "Layer Max", etc
                1,                              // 0, 1, etc
                KMAX_,                 		// 24, 72, etc
                NULL,       /* Julian start date for time steps 960412 SRT */
                NULL,       /* Julian start time for time steps 960412 SRT */
                &minLevel_,                     // current Min
                &maxLevel_,                     // current Max
                valsModifiedParentCB,           // call this when changed
                (void *) this,                  // this object
                estring                         // to hold error msgs
                );
	}
}



Level::~Level()           		// Destructor
{
if (levelDialogBox_) delete levelDialogBox_;
}



int Level::match(void *target) 		// override baseType's 
                                        // virtual match() 
{
int *ip = (int *) target;

return (*ip == KMAX_);
}



char *Level::getClassName(void)         // override baseType's 
                                        // virtual getClassName()
{
static char *myName = "Level";
return myName;
}



void Level::print(FILE *output) 	// override linkedList's print()
{
fprintf(output, "-----------\n");
fprintf(output, "Level print\n");
fprintf(output, "-----------\n");
fprintf(output, "KMAX_ == %d\n", KMAX_);
fprintf(output, "minLevel_ == %d\n", minLevel_);
fprintf(output, "maxLevel_ == %d\n", maxLevel_);
}



int *Level::getCopyOfLevels(void)	// to supply a copy of its data in array format -
					// 0 for off, 1 for on
{
int i, *ans;

ans = (int *) malloc(KMAX_*sizeof(int));
if (ans != NULL)
	for (i = 0; i < KMAX_; i++)
		ans[i] =  ((i+1 >= minLevel_) && (i+1 <= maxLevel_));

return ans;
}



int Level::showUI(void)			// brings up a UI which allows
					// the user to modify the layers
					// selected
{
if (levelDialogBox_)
        levelDialogBox_->postOptionDialog();

#ifdef DIAGNOSTICS
else
if (KMAX_ == 1)
	fprintf(stderr, "Can't select min/max layers when there is only one layer\n");
else
        fprintf(stderr, "No levelDialogBox_ to edit with "
                "in Level::showUI()!\n"); // SRT
#endif // DIAGNOSTICS

return 0;
}


int Level::setMinAndMaxLevelTo(int l)
{
if ((l >= 1) && (l <= get_nLevels()))
	levelDialogBox_->setMinMax(l, l);

return 0;
}


int Level::setLevelRange(int lmin, int lmax)
{
int t;

if (lmin > lmax)
	{
	t = lmin;
	lmin = lmax;
	lmax = t;
	}

if ((lmin >= 1) && (lmin <= get_nLevels()))
   if ((lmax >= 1) && (lmax <= get_nLevels()))
	levelDialogBox_->setMinMax(lmin, lmax);

return 0;
}




static void modifiedTheseLevelsCB(void *obj)   // run this if user subselects
                                               // on this Level's range
{
Level *lev = (Level *) obj;
Formula *formula;

        // NOTIFY EACH FORMULA THAT WE'VE CHANGED A LEVEL RANGE
formula = (Formula *) lev->get_formula_list()->head();
while(formula)
        {
#ifdef DIAGNOSTICS
	fprintf(stderr, "Level::modifiedTheseLevelsCB() about to call "
                "formula(%s)->levelsWereChanged(%d)\n", formula->getFormulaName(), lev->get_nLevels());
#endif // DIAGNOSTICS

        formula->levelsWereChanged(lev->get_nLevels());
        formula = (Formula *) lev->get_formula_list()->next();
        }
}
