/****************************************************************************
 *
 * Project Title: Environmental Decision Support System
 *
 *         File: @(#)LocalFileBrowser.cc	2.1
 *     Pathname: /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.LocalFileBrowser.cc
 * Last updated: 12/15/97 16:33:16
 *
 * COPYRIGHT (C) 1996, University of North Carolina at Chapel Hill
 * Hill under terms of the GNU Public License.  See gpl.txt for more details.
 *
 * See file COPYRIGHT for license information on this and supporting software.
 *
 * Carolina Environmental Program
 * University of North Carolina at Chapel Hill
 * 137 E. Franklin St.
 * Chapel Hill, NC 27599-6116
 *
 * See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
 *
 ****************************************************************************/

static const char *SCCS_ID = "@(#)LocalFileBrowser.cc	2.1 /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.LocalFileBrowser.cc 12/15/97 16:33:16";

//////////////////////////////////////////////////////////////////////////////
// File:	LocalFileBrowser.cc
// Author:	Steve Thorpe
// Date:	May 22, 1996
///////////////////////////////////////////////////////////////////////////////
//
// Modification History
// When   Who What
// 960522 SRT Implemented
//
//////////////////////////////////////////////////////////////////////////////

#include "LocalFileBrowser.h"



////////////////////////////////////////////////
//
//	LocalFileBrowser()
//
////////////////////////////////////////////////

LocalFileBrowser::LocalFileBrowser
	( 
	Widget parent, 
	char *fileTypeName,
        char *defaultDirectory, 
	void *gotSaveFileName, /* void (*gotSaveFileName)(void *, char *), */
        void *gotLoadFileName, /* void (*gotLoadFileName)(void *, char *), */
	void *object
	)
{
   defaultDirectory_ = NULL;
   fileTypeName_ = NULL;
   load_selection_dialog_ = NULL;
   save_selection_dialog_ = NULL;
   parent_ = parent;
   if (fileTypeName) fileTypeName_ = strdup(fileTypeName); 
   if (defaultDirectory) defaultDirectory_ = strdup(defaultDirectory); 
   gotSaveFileName_ = (void (*)(void *, char *)) gotSaveFileName;
   gotLoadFileName_ = (void (*)(void *, char *)) gotLoadFileName;
   object_ = object;
}



////////////////////////////////////////////////
//
//	~LocalFileBrowser()
//
////////////////////////////////////////////////

LocalFileBrowser::~LocalFileBrowser()
{
   if (fileTypeName_) free(fileTypeName_); fileTypeName_ = NULL;
   if (defaultDirectory_) free(defaultDirectory_); defaultDirectory_ = NULL;
}



////////////////////////////////////////////////
//
//	postLoadSelectionDialog()
//
////////////////////////////////////////////////

void LocalFileBrowser::postLoadSelectionDialog()
{
   if (load_selection_dialog_ == NULL)
	createLoadSelectionDialog();
   if (XtIsManaged(load_selection_dialog_)) XtUnmanageChild(load_selection_dialog_); XtManageChild(load_selection_dialog_);
} 



////////////////////////////////////////////////
//
//	postSaveSelectionDialog()
//
////////////////////////////////////////////////

void LocalFileBrowser::postSaveSelectionDialog()
{
   if (save_selection_dialog_ == NULL)
	createSaveSelectionDialog();
   if (XtIsManaged(save_selection_dialog_)) XtUnmanageChild(save_selection_dialog_); XtManageChild(save_selection_dialog_);
}


 

////////////////////////////////////////////////
//
//	createSaveSelectionDialog()
//
////////////////////////////////////////////////

void LocalFileBrowser::createSaveSelectionDialog()
{

   char strbuf[100];
   XmString dir = defaultDirectory_ ? 
		  XmStringCreateLocalized(defaultDirectory_) :
		  XmStringCreateLocalized(getenv("HOME"));

   sprintf(strbuf, "Save %s as:", fileTypeName_ ? fileTypeName_ : "");
   save_selection_dialog_ = 
	XmCreateFileSelectionDialog(parent_, strbuf, NULL, 0);
   XtVaSetValues(save_selection_dialog_,
                XmNautoUnmanage,        True,
   		XmNdirectory,           dir, // SRT 960425
                NULL);

   XmStringFree(dir); dir = NULL; // SRT 960425

   XtAddCallback(save_selection_dialog_, XmNokCallback, &LocalFileBrowser::save_okCB, (XtPointer) this);
}



////////////////////////////////////////////////
//
//	save_okCB()
//
////////////////////////////////////////////////

void LocalFileBrowser::save_okCB(Widget, XtPointer clientData, XtPointer callData)
{
   LocalFileBrowser *obj = (LocalFileBrowser *) clientData;
   XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *) callData;

   obj->save_ok_cb(cbs->value);
}



////////////////////////////////////////////////
//
//	save_ok_cb()
//
////////////////////////////////////////////////

void LocalFileBrowser::save_ok_cb(XmString xmstr)
{
   char *str = NULL;

   if (XmStringGetLtoR(xmstr, XmSTRING_DEFAULT_CHARSET, &str))
	{
	XtUnmanageChild(save_selection_dialog_);
	(*gotSaveFileName_)(object_, str);
	}

   XtFree(str);
}



////////////////////////////////////////////////
//
//	createLoadSelectionDialog()
//
////////////////////////////////////////////////

void LocalFileBrowser::createLoadSelectionDialog()
{
   char strbuf[100];
   XmString dir = defaultDirectory_ ? 
		  XmStringCreateLocalized(defaultDirectory_) :
		  XmStringCreateLocalized(getenv("HOME"));

   sprintf(strbuf, "Load %s", fileTypeName_ ? fileTypeName_ : "");

   load_selection_dialog_ = XmCreateFileSelectionDialog(parent_, strbuf, NULL, 0);
   XtVaSetValues(load_selection_dialog_,
		XmNautoUnmanage,	True,
                XmNdirectory,           dir, // SRT 960425
		NULL);

   XmStringFree(dir); dir = NULL; // SRT 960425

   XtAddCallback(load_selection_dialog_, XmNokCallback, &LocalFileBrowser::load_okCB, (XtPointer) this);
}



////////////////////////////////////////////////
//
//	load_okCB()
//
////////////////////////////////////////////////

void LocalFileBrowser::load_okCB(Widget, XtPointer clientData, XtPointer callData)
{
   LocalFileBrowser *obj = (LocalFileBrowser *) clientData;
   XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *) callData;

   obj->load_ok_cb(cbs->value);
}


////////////////////////////////////////////////
//
//	load_ok_cb()
//
////////////////////////////////////////////////

void LocalFileBrowser::load_ok_cb(XmString xmstr)
{
   char *str;

   if (XmStringGetLtoR(xmstr, XmSTRING_DEFAULT_CHARSET, &str))
	{
	XtUnmanageChild(load_selection_dialog_);
	(*gotLoadFileName_)(object_, str);
	}

   XtFree(str);
}



