/****************************************************************************
 *
 * Project Title: Environmental Decision Support System
 *
 *         File: @(#)Menus.cc	2.1
 *     Pathname: /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.Menus.cc
 * Last updated: 12/15/97 16:33:23
 *
 * Made available by MCNC and the Carolina Environmental Program of UNC Chapel
 * Hill under terms of the GNU Public License.  See gpl.txt for more details.
 *
 * See file COPYRIGHT for license information on this and supporting software.
 *
 * Carolina Environmental Program
 * University of North Carolina at Chapel Hill
 * 137 E. Franklin St.
 * Chapel Hill, NC 27599-6116
 *
 * See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
 *
 ****************************************************************************/

static const char *SCCS_ID = "@(#)Menus.cc	2.1 /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.Menus.cc 12/15/97 16:33:23";

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// Menus.C 
// Kah Eng Pua
// December 7, 1994
// Note:
//	The createMenuButtons function is based on Douglas Young's original 
//	menus.c program.  See pp 104 to 106 of The X Window System 
//	Programming and Applications with Xt.  Prentice Hall, 1990.
//	Modification by K. Eng Pua
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#include "Menus.h"

Menus::Menus()
{
   // Empty constructor
}


void Menus::createMenuBar(Widget parent, XtPointer caller)
{
   caller_ = caller;

   assert(parent);

   menuBar_ = XmCreateMenuBar(parent, "menubar", NULL, 0);
   XtVaSetValues(menuBar_,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);
   if (XtIsManaged(menuBar_)) XtUnmanageChild(menuBar_); XtManageChild(menuBar_);

}


void Menus::createMenuButtons(char *title, Widget menu, menu_struct *menulist, int nitems)
{
   /*
   ** If a title is given, create Label and Separator widgets.
   */
   if (title) {
      XtCreateManagedWidget(title, xmLabelWidgetClass, menu, NULL, 0);
      XtCreateManagedWidget("separator", xmSeparatorWidgetClass,
                            menu, NULL, 0);
   }
   /*
   **  Create an entry for each item in the menu.
   */
   for (int i=0; i<nitems; i++) {
      /*
      ** A NULL name represents a separator.
      */
      if (menulist[i].name == NULL) {
         XtCreateManagedWidget("separator",
                               xmSeparatorWidgetClass,
                               menu, NULL, 0);
      }
      /*
      ** If there is a name and a callback, create a selectable 
      ** menu entry and register the callback function.
      */
      else if (menulist[i].func) {
         menulist[i].wid = XtCreateManagedWidget(menulist[i].name,
                                       xmPushButtonWidgetClass,
                                       menu, NULL, 0);
         XtAddCallback(menulist[i].wid, XmNactivateCallback,
                       menulist[i].func, caller_);
      }
      /*
      ** If there is a name, but no callback function, the entry
      ** must be a label, unless there is a submenu.
      */
      else if (!menulist[i].sub_menu)
         menulist[i].wid = XtCreateManagedWidget(menulist[i].name,
                                            xmLabelWidgetClass,
                                            menu, NULL, 0);
      /*
      ** If we got here, the entry must be a submenu.
      ** Create a pulldown menu pane and an XmCascadeButton
      ** widget.  Attach the menu pane and make a recursive call
      ** to create the entries in the submenu.
      */
      else {
         Widget sub_menu = XmCreatePulldownMenu(menu,
                                         menulist[i].name,
                                         NULL, 0);
         menulist[i].wid = XtVaCreateManagedWidget(menulist[i].name,
			xmCascadeButtonWidgetClass, menu, 
			XmNsubMenuId, sub_menu,
			NULL);
         createMenuButtons(menulist[i].sub_menu_title,
                                   sub_menu, menulist[i].sub_menu,
                                   menulist[i].n_sub_items);
      }
   }
}
